package dk.apaq.crud;

import dk.apaq.filter.Filter;
import dk.apaq.filter.limit.Limit;
import dk.apaq.filter.sort.Sorter;
import java.util.List;

/**
 * An Interface for a Crud. The interface is split into several interfaces allowing
 * for Crud's to only implement some methods fx. for readonly cruds.
 *
 * If more functionality is required, fx. update, create, delete or filtering, then
 * the cruds need to implement the subinterfaces.
 * @param <IDT> The type used as Id for the entity, fx. String.
 * @param <BT> The entity type.
 */
public interface Crud<IDT,T extends HasId<IDT>> {

    /**
     * Reads a single enntity by its id.
     * @param id The id of the entity to read.
     * @return Returns the entity or null if entity available with the specified id.
     */
    T read(IDT id);

    /**
     * Lists all id's.
     * @return The list of all id's
     */
    List<IDT> listIds();

    /**
     * Lists all id's.
     * @param limit Limits the returned result
     * @return The list of all id's
     */
    List<IDT> listIds(Limit limit);


    /**
     * Interface for editable cruds.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Editable<IDT,BT extends HasId<IDT>> extends Crud<IDT,BT> {
        BT create();
        void update(BT entity);
        void delete(IDT id);
    }

    /**
     * Interface for filterable cruds.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Filterable<IDT,BT extends HasId<IDT>> extends Crud<IDT,BT> {
        List<IDT> listIds(Filter filter, Sorter sorter);
        List<IDT> listIds(Filter filter, Sorter sorter, Limit limit);
    }

    /**
     * Interface for cruds that implements all crud methods.
     * @param <IDT> The type used as Id for the entity, fx. String.
     * @param <BT> The entity type.
     */
    public interface Complete<IDT,BT extends HasId<IDT>> extends Filterable<IDT,BT>, Editable<IDT, BT> { }
}
