package dk.apaq.crud.core;

import dk.apaq.crud.Crud;
import dk.apaq.crud.CrudEvent;
import dk.apaq.crud.CrudListener;
import dk.apaq.crud.CrudNotifier;
import dk.apaq.crud.HasId;
import java.util.ArrayList;
import java.util.List;

/**
 * An abstract implementation of a <code>Crud.Editable</code> instance taking care
 * of CrudNotifer logic. An implementing class needs only to call the fire* methods(fx. fireOnRead)
 * and this class will make sure to fire evetns on the listeners.
 */
public abstract class BaseCrud<IDT, BT extends HasId<IDT>> implements
        Crud.Editable<IDT, BT>, CrudNotifier {

    private final List<CrudListener<IDT, BT>> listeners = new ArrayList<CrudListener<IDT, BT>>();

    @Override
    public void addListener(CrudListener listener) {
        listeners.add(listener);
    }

    @Override
    public void removeListener(CrudListener listener) {
        listeners.remove(listener);
    }

    /**
     * Tells listeners that an entity has been read from crud.
     * @param entity The entity that was read.
     */
    protected void fireOnRead(BT entity) {
        CrudEvent.WithEntity e = createEvent(entity);
        for (CrudListener listener : listeners) {
            listener.onEntityRead(e);
        }
    }

    /**
     * Tells listeners that an entity has been created.
     * @param entity The entity that was created.
     */
    protected void fireOnCreate(BT entity) {
        CrudEvent.WithEntity e = createEvent(entity);

        for (CrudListener listener : listeners) {
            listener.onEntityCreate(e);
        }
    }

    /**
     * Tells listeners that an entity was updated.
     * @param entity The entity that was updated.
     */
    protected void fireOnUpdate(BT entity) {
        CrudEvent.WithEntity e = createEvent(entity);

        for (CrudListener listener : listeners) {
            listener.onEntityUpdate(e);
        }
    }

    /**
     * Tells listeners that an entity was deleted.
     * @param id The id of the entity that was deleted.
     */
    protected void fireOnDelete(IDT id) {
        CrudEvent.WithId e = createEvent(id);
        for (CrudListener listener : listeners) {
            listener.onEntityDelete(e);
        }
    }

    /**
     * Tells listeners that an entity is about tobe read.
     * @param id The id of the entity that is to be read.
     */
    protected void fireOnBeforeRead(IDT id) {
        CrudEvent.WithId e = createEvent(id);
        for (CrudListener listener : listeners) {
            listener.onBeforeEntityRead(e);
        }
    }

    /**
     * Tells listeners that a new entity is about to be created.
     */
    protected void fireOnBeforeCreate() {
        CrudEvent e = createEvent();

        for (CrudListener listener : listeners) {
            listener.onBeforeEntityCreate(e);
        }
    }

    /**
     * Tells listeners that an entity is about to be updated.
     * @param entity The entity that holds changes and is about to be persisted.
     */
    protected void fireOnBeforeUpdate(BT entity) {
        CrudEvent.WithEntity e = createEvent(entity);

        for (CrudListener listener : listeners) {
            listener.onBeforeEntityUpdate(e);
        }
    }

    /**
     * Tells listeners that an eneity is about to be deleted.
     * @param id The id of the entity that is about to be deleted.
     */
    protected void fireOnBeforeDelete(IDT id) {
        CrudEvent.WithId e = createEvent(id);
        for (CrudListener listener : listeners) {
            listener.onBeforeEntityDelete(e);
        }
    }

    private CrudEvent createEvent() {
        return new CrudEvent(this);
    }

    private CrudEvent.WithId<IDT, BT> createEvent(IDT id) {
        return new CrudEvent.WithId<IDT, BT>(this, id);
    }

    private CrudEvent.WithEntity<IDT, BT> createEvent(BT entity) {
        return new CrudEvent.WithEntity<IDT, BT>(this, entity);
    }

}
